﻿/*
 * Project Blend: the Blossom of .NET Development
 * Copyright (c) 2007 Project Blend
 * 
 * 이 라이브러리는 자유 소프트웨어입니다. 소프트웨어의 피양도자는 자유 소프트웨어 재단이
 * 공표한 GNU 약소 일반 공중 사용 허가서 2.1판 또는 그 이후 판을 임의로 선택해서, 그 규정에
 * 따라 라이브러리를 개작하거나 재배포할 수 있습니다.
 * 
 * 이 라이브러리는 유용하게 사용될 수 있으리라는 희망에서 배포되고 있지만, 특정한 목적에
 * 맞는 적합성 여부나 판매용으로 사용할 수 있으리라는 묵시적인 보증을 포함한 어떠한 형태의
 * 보증도 제공하지 않습니다. 보다 자세한 사항에 대해서는 GNU 약소 일반 공중 사용 허가서를
 * 참고하시기 바랍니다.
 * 
 * GNU 약소 일반 공중 사용 허가서는 이 라이브러리와 함께 제공됩니다. 만약, 이 문서가 누락
 * 되어있다면 자유 소프트웨어 재단으로 문의하시기 바랍니다.
 * 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * Korean Translation Comes from: http://korea.gnu.org/people/chsong/copyleft/lgpl.ko.html
 * */

using System;
using System.ComponentModel;
using System.Collections.Generic;

namespace Blend
{
    [Serializable()]
    public class NewQuickSort<T> : SortBase<T>
        where T : IComparable<T>, IComparable
    {
        [NonSerialized()]
        public const int DefaultQuickSortStackSize = 64;

        public NewQuickSort()
            : base()
        {
        }

        public NewQuickSort(int nInternalStackSize)
            : this()
        {
            this.InternalStackSize = nInternalStackSize;
        }

        public NewQuickSort(IList<T> oList)
            : base(oList)
        {
        }

        public NewQuickSort(int nInternalStackSize, IList<T> oList)
            : this(oList)
        {
            this.InternalStackSize = nInternalStackSize;
        }

        public NewQuickSort(params T[] oArray)
            : base(oArray)
        {
        }

        public NewQuickSort(int nInternalStackSize, params T[] oArray)
            : this(oArray)
        {
            this.InternalStackSize = nInternalStackSize;
        }

        private int m_nInternalStackSize = DefaultQuickSortStackSize;

        [Browsable(true)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public int InternalStackSize
        {
            get { return this.m_nInternalStackSize; }
            set { this.m_nInternalStackSize = (value < DefaultQuickSortStackSize) ? DefaultQuickSortStackSize : value; }
        }

        public override unsafe void SortInternal(bool reverse)
        {
            if (this.Count < 1)
                return;

            int lo = 0, hi = this.Count - 1, mid, loguy, higuy, stkptr = 0;
            int* lostk = stackalloc int[this.m_nInternalStackSize];
            int* histk = stackalloc int[this.m_nInternalStackSize];
            T temp;

            while (true)
            {
                mid = lo + (((hi - lo) + 1) / 2);

                temp = this[mid];
                this[mid] = this[lo];
                this[lo] = temp;

                loguy = lo;
                higuy = hi + 1;

                while (true)
                {
                    do loguy++;
                    while (loguy <= hi && !this.InternalComparer(this[loguy], this[lo], reverse, true));

                    do higuy--;
                    while (higuy > lo && this.InternalComparer(this[higuy], this[lo], reverse, true));

                    if (higuy < loguy)
                        break;

                    temp = this[loguy];
                    this[loguy] = this[higuy];
                    this[higuy] = temp;
                }

                temp = this[lo];
                this[lo] = this[higuy];
                this[higuy] = temp;

                if (higuy - 1 - lo >= hi - loguy)
                {
                    if (lo + 1 < higuy)
                    {
                        lostk[stkptr] = lo;
                        histk[stkptr] = higuy - 1;
                        ++stkptr;
                    }

                    if (loguy < hi)
                    {
                        lo = loguy;
                        continue;
                    }
                }
                else
                {
                    if (loguy < hi)
                    {
                        lostk[stkptr] = loguy;
                        histk[stkptr] = hi;
                        ++stkptr;
                    }

                    if (lo + 1 < higuy)
                    {
                        hi = higuy - 1;
                        continue;
                    }
                }

                --stkptr;

                if (stkptr >= 0)
                {
                    lo = lostk[stkptr];
                    hi = histk[stkptr];
                }
                else
                    break;
            }
        }
    }
}
