﻿/*
 * Project Blend: the Blossom of .NET Development
 * Copyright (c) 2007 Project Blend
 * 
 * 이 라이브러리는 자유 소프트웨어입니다. 소프트웨어의 피양도자는 자유 소프트웨어 재단이
 * 공표한 GNU 약소 일반 공중 사용 허가서 2.1판 또는 그 이후 판을 임의로 선택해서, 그 규정에
 * 따라 라이브러리를 개작하거나 재배포할 수 있습니다.
 * 
 * 이 라이브러리는 유용하게 사용될 수 있으리라는 희망에서 배포되고 있지만, 특정한 목적에
 * 맞는 적합성 여부나 판매용으로 사용할 수 있으리라는 묵시적인 보증을 포함한 어떠한 형태의
 * 보증도 제공하지 않습니다. 보다 자세한 사항에 대해서는 GNU 약소 일반 공중 사용 허가서를
 * 참고하시기 바랍니다.
 * 
 * GNU 약소 일반 공중 사용 허가서는 이 라이브러리와 함께 제공됩니다. 만약, 이 문서가 누락
 * 되어있다면 자유 소프트웨어 재단으로 문의하시기 바랍니다.
 * 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * Korean Translation Comes from: http://korea.gnu.org/people/chsong/copyleft/lgpl.ko.html
 * */

using System;
using System.Collections.Generic;

namespace Blend.Sortings
{
    [Serializable()]
    public class JSort<T> : SortBase<T>
        where T: IComparable<T>, IComparable
    {
        public JSort()
            : base()
        {
        }

        public JSort(IList<T> oList)
            : base(oList)
        {
        }

        public JSort(params T[] oArray)
            : base(oArray)
        {
        }

        protected void Reheap(int length, int i, bool reverse)
        {
            bool done = false;
            T obj = this[i];
            int parent = i;
            int child = 2 * (i + 1) - 1;

            while ((child < length) && (!done))
            {
                if (child < length - 1)
                    if (!this.InternalComparer(this[child], this[child + 1], reverse))
                        child++;

                if (this.InternalComparer(obj, this[child], reverse))
                    done = true;
                else
                {
                    this[parent] = this[child];
                    parent = child;
                    child = 2 * (parent + 1) - 1;
                }
            }

            this[parent] = obj;
        }

        protected void InverseReheap(int length, int i, bool reverse)
        {
            bool done = false;
            T obj = this[length - 1 - i];
            int parent = i;
            int child = 2 * (i + 1) - 1;

            while ((child < length) && (!done))
            {
                if (child < length - 1)
                    if (this.InternalComparer(this[length - 1 - child], this[length - 1 - (child + 1)], reverse))
                        child++;

                if (!this.InternalComparer(obj, this[length - 1 - child], reverse))
                    done = true;
                else
                {
                    this[length - 1 - parent] = this[length - 1 - child];
                    parent = child;
                    child = 2 * (parent + 1) - 1;
                }
            }

            this[length - 1 - parent] = obj;
        }

        protected override void SortInternal(bool reverse)
        {
            for (int i = this.Count - 1; i >= 0; i--)
                this.Reheap(this.Count, i, reverse);

            for (int i = this.Count - 1; i >= 0; i--)
                this.InverseReheap(this.Count, 1, reverse);

            for (int j = 1; j < this.Count; j++)
            {
                T obj = this[j];
                int i = (j - 1);

                while (i >= 0 && !this.InternalComparer(this[i], obj, reverse))
                {
                    this[i + 1] = this[i];
                    i--;
                }

                this[i + 1] = obj;
            }
        }
    }
}
