﻿/*
 * Project Blend: the Blossom of .NET Development
 * Copyright (c) 2007 Project Blend
 * 
 * 이 라이브러리는 자유 소프트웨어입니다. 소프트웨어의 피양도자는 자유 소프트웨어 재단이
 * 공표한 GNU 약소 일반 공중 사용 허가서 2.1판 또는 그 이후 판을 임의로 선택해서, 그 규정에
 * 따라 라이브러리를 개작하거나 재배포할 수 있습니다.
 * 
 * 이 라이브러리는 유용하게 사용될 수 있으리라는 희망에서 배포되고 있지만, 특정한 목적에
 * 맞는 적합성 여부나 판매용으로 사용할 수 있으리라는 묵시적인 보증을 포함한 어떠한 형태의
 * 보증도 제공하지 않습니다. 보다 자세한 사항에 대해서는 GNU 약소 일반 공중 사용 허가서를
 * 참고하시기 바랍니다.
 * 
 * GNU 약소 일반 공중 사용 허가서는 이 라이브러리와 함께 제공됩니다. 만약, 이 문서가 누락
 * 되어있다면 자유 소프트웨어 재단으로 문의하시기 바랍니다.
 * 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * Korean Translation Comes from: http://korea.gnu.org/people/chsong/copyleft/lgpl.ko.html
 * */

using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace Blend.Sortings
{
    /// <summary>
    /// 정렬 알고리즘 클래스들의 기본 형식입니다.
    /// </summary>
    /// <typeparam name="T">컬렉션 원소 형식을 지정합니다. 비교 가능한 형식이어야 합니다.</typeparam>
    [Serializable()]
    public abstract class SortBase<T> : Collection<T>
        where T : IComparable<T>, IComparable
    {
        public SortBase()
            : base()
        {
        }

        public SortBase(IList<T> oList)
            : base(oList)
        {
        }

        public SortBase(params T[] oArray)
            : this(new List<T>(oArray))
        {
        }

        public event EventHandler BeforeSort;
        public event EventHandler AfterSort;

        [NonSerialized()]
        private bool enableEvent = false;

        protected virtual void OnBeforeSort()
        {
            if (this.BeforeSort != null)
                this.BeforeSort(this, EventArgs.Empty);
        }

        protected virtual void OnAfterSort()
        {
            if (this.AfterSort != null)
                this.AfterSort(this, EventArgs.Empty);
        }

        protected abstract void SortInternal(bool descending);

        /// <summary>
        /// 오른쪽 항목이 왼쪽 항목보다 큰지 비교합니다.
        /// </summary>
        /// <param name="comparableLeft"></param>
        /// <param name="comparableRight"></param>
        /// <param name="descending"></param>
        /// <returns></returns>
        protected bool InternalComparer(T comparableLeft, T comparableRight, bool descending)
        {
            return this.InternalComparer(comparableLeft, comparableRight, descending, true);
        }

        /// <summary>
        /// 오른쪽 항목이 왼쪽 항목보다 큰지 비교합니다.
        /// </summary>
        /// <param name="comparableLeft"></param>
        /// <param name="comparableRight"></param>
        /// <param name="descending"></param>
        /// <param name="includeEquality"></param>
        /// <returns></returns>
        protected bool InternalComparer(T comparableLeft, T comparableRight, bool descending, bool includeEquality)
        {
            return descending ?
                (includeEquality ? (comparableLeft.CompareTo(comparableRight) <= 0) : (comparableRight.CompareTo(comparableRight) < 0)) :
                (includeEquality ? (comparableLeft.CompareTo(comparableRight) >= 0) : (comparableRight.CompareTo(comparableRight) > 0));
        }

        public void Sort(bool descending)
        {
            if (this.Count < 1)
                return;

            if (this.enableEvent)
                this.OnBeforeSort();

            this.SortInternal(descending);

            if (this.enableEvent)
                this.OnAfterSort();
        }

        public void Sort()
        {
            this.Sort(false);
        }

        public void SortDescending()
        {
            this.Sort(true);
        }

        public bool EnableEvent
        {
            get { return this.enableEvent; }
            set { this.enableEvent = value; }
        }
    }
}
