﻿/*
 * Project Blend: the Blossom of .NET Development
 * Copyright (c) 2007 Project Blend
 * 
 * 이 라이브러리는 자유 소프트웨어입니다. 소프트웨어의 피양도자는 자유 소프트웨어 재단이
 * 공표한 GNU 약소 일반 공중 사용 허가서 2.1판 또는 그 이후 판을 임의로 선택해서, 그 규정에
 * 따라 라이브러리를 개작하거나 재배포할 수 있습니다.
 * 
 * 이 라이브러리는 유용하게 사용될 수 있으리라는 희망에서 배포되고 있지만, 특정한 목적에
 * 맞는 적합성 여부나 판매용으로 사용할 수 있으리라는 묵시적인 보증을 포함한 어떠한 형태의
 * 보증도 제공하지 않습니다. 보다 자세한 사항에 대해서는 GNU 약소 일반 공중 사용 허가서를
 * 참고하시기 바랍니다.
 * 
 * GNU 약소 일반 공중 사용 허가서는 이 라이브러리와 함께 제공됩니다. 만약, 이 문서가 누락
 * 되어있다면 자유 소프트웨어 재단으로 문의하시기 바랍니다.
 * 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * Korean Translation Comes from: http://korea.gnu.org/people/chsong/copyleft/lgpl.ko.html
 * */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace Blend.xPlatform
{
    [Serializable]
    public class AddressWalker64 : IEnumerable<IntPtr>
    {
        public AddressWalker64(short address)
            : this((int)address)
        {
        }

        public AddressWalker64(ushort address)
            : this((uint)address)
        {
        }

        public AddressWalker64(int address)
            : this((long)address)
        {
        }

        public AddressWalker64(uint address)
            : this((long)address)
        {
        }

        public AddressWalker64(long address)
        {
            this.Address = address;
        }

        public AddressWalker64(IntPtr address)
            : this(address.ToInt32())
        {
        }

        public unsafe AddressWalker64(void* pointer)
            : this(new IntPtr(pointer).ToInt32())
        {
        }

        public AddressWalker64(AddressWalker32 walker)
            : this(walker.Address)
        {
        }

        public AddressWalker64(AddressWalker64 walker)
            : this(walker.address)
        {
        }

        private long address = 0L;

        [NonSerialized]
        private int frameSize = 0;

        public long Address
        {
            get { return this.address; }
            set { this.address = (value < 0L ? value * (-1) : value); }
        }

        public int FrameSize
        {
            get { return this.frameSize; }
            set { this.frameSize = (value < 0 ? value * (-1) : value); }
        }

        public bool IsAnyPointer
        {
            get { return this.frameSize.Equals(0); }
        }

        public bool IsNullPointer
        {
            get { return this.address.Equals(0L); }
        }

        public bool SelectFrameSize(Type targetType)
        {
            try
            {
                this.frameSize = Marshal.SizeOf(targetType);
                return true;
            }
            catch { return false; }
        }

        public IntPtr this[int frameIndex]
        {
            get { return (this + frameIndex); }
        }

        public T GetCurrent<T>()
        {
            return InteropServices.ProduceManagedObject<T>(this);
        }

        public unsafe override bool Equals(object obj)
        {
            if (obj is short)
                return this.Equals((short)obj);
            else if (obj is ushort)
                return this.Equals((ushort)obj);
            else if (obj is int)
                return this.Equals((int)obj);
            else if (obj is uint)
                return this.Equals((uint)obj);
            else if (obj is long)
                return this.Equals((long)obj);
            else if (obj is IntPtr)
                return this.Equals((IntPtr)obj);
            else if (obj is AddressWalker64)
                return this.Equals((AddressWalker64)obj);
            else
                return base.Equals(obj);
        }

        public bool Equals(short address)
        {
            return this.address.Equals((int)address);
        }

        public bool Equals(ushort address)
        {
            return this.address.Equals((int)address);
        }

        public bool Equals(int address)
        {
            return this.address.Equals(address);
        }

        public bool Equals(uint address)
        {
            return this.address.Equals(address);
        }

        public bool Equals(long address)
        {
            return this.address.Equals(address);
        }

        public bool Equals(IntPtr address)
        {
            return this.address.Equals(address.ToInt32());
        }

        public unsafe bool Equals(void* pointer)
        {
            return this.address.Equals(new IntPtr(pointer).ToInt32());
        }

        public bool Equals(AddressWalker32 walker)
        {
            return this.address.Equals(walker.Address);
        }

        public bool Equals(AddressWalker64 walker)
        {
            return this.address.Equals(walker.address);
        }

        public override int GetHashCode()
        {
            return this.address.GetHashCode();
        }

        public override string ToString()
        {
            return new IntPtr(this.address).ToString();
        }

        public string ToString(string format)
        {
            return new IntPtr(this.address).ToString(format);
        }

        public static AddressWalker64 operator +(AddressWalker64 left, int elementCount)
        {
            int temp = left.frameSize * elementCount;

            return new AddressWalker64(
                (left.address + temp < 0 ? 0 : left.address + temp));
        }

        public static AddressWalker64 operator -(AddressWalker64 left, int elementCount)
        {
            int temp = left.frameSize * elementCount;

            return new AddressWalker64(
                (left.address - temp < 0 ? 0 : left.address - temp));
        }

        public static AddressWalker64 operator ++(AddressWalker64 left)
        {
            left.address += left.frameSize;
            return left;
        }

        public static AddressWalker64 operator --(AddressWalker64 left)
        {
            if (left.address - left.frameSize < 0)
                left.address = 0;
            else
                left.address -= left.frameSize;

            return left;
        }

        public static bool operator ==(AddressWalker64 left, long right)
        {
            return left.Equals(right);
        }

        public static bool operator ==(AddressWalker64 left, int right)
        {
            return left.Equals(right);
        }

        public static bool operator ==(AddressWalker64 left, uint right)
        {
            return left.Equals(right);
        }

        public static bool operator ==(AddressWalker64 left, short right)
        {
            return left.Equals(right);
        }

        public static bool operator ==(AddressWalker64 left, ushort right)
        {
            return left.Equals(right);
        }

        public static bool operator ==(AddressWalker64 left, IntPtr right)
        {
            return left.Equals(right);
        }

        public static unsafe bool operator ==(AddressWalker64 left, void* right)
        {
            return left.Equals(right);
        }

        public static bool operator ==(AddressWalker64 left, AddressWalker32 right)
        {
            return left.Equals(right);
        }

        public static bool operator ==(AddressWalker64 left, AddressWalker64 right)
        {
            return left.Equals(right);
        }

        public static bool operator !=(AddressWalker64 left, long right)
        {
            return !left.Equals(right);
        }

        public static bool operator !=(AddressWalker64 left, int right)
        {
            return !left.Equals(right);
        }

        public static bool operator !=(AddressWalker64 left, uint right)
        {
            return !left.Equals(right);
        }

        public static bool operator !=(AddressWalker64 left, short right)
        {
            return !left.Equals(right);
        }

        public static bool operator !=(AddressWalker64 left, ushort right)
        {
            return !left.Equals(right);
        }

        public static bool operator !=(AddressWalker64 left, IntPtr right)
        {
            return !left.Equals(right);
        }

        public static unsafe bool operator !=(AddressWalker64 left, void* right)
        {
            return !left.Equals(right);
        }

        public static bool operator !=(AddressWalker64 left, AddressWalker32 right)
        {
            return !left.Equals(right);
        }

        public static bool operator !=(AddressWalker64 left, AddressWalker64 right)
        {
            return !left.Equals(right);
        }

        public static implicit operator long(AddressWalker64 walker)
        {
            return walker.address;
        }

        public static implicit operator IntPtr(AddressWalker64 walker)
        {
            return new IntPtr(walker.address);
        }

        public static implicit operator int(AddressWalker64 walker)
        {
            return new IntPtr(walker.address).ToInt32();
        }

        public static unsafe implicit operator void*(AddressWalker64 walker)
        {
            return new IntPtr(walker.address).ToPointer();
        }

        public static implicit operator AddressWalker64(long address)
        {
            return new AddressWalker64(address);
        }

        public static implicit operator AddressWalker64(IntPtr address)
        {
            return new AddressWalker64(address);
        }

        public static unsafe implicit operator AddressWalker64(void* pointer)
        {
            return new AddressWalker64(pointer);
        }

        public static implicit operator AddressWalker64(AddressWalker32 walker)
        {
            return new AddressWalker64(walker);
        }

        public IEnumerator<IntPtr> GetEnumerator()
        {
            return new Mem64Enumerator(this);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
    }
}
