﻿/*
 * Project Blend: the Blossom of .NET Development
 * Copyright (c) 2007 Project Blend
 * 
 * 이 라이브러리는 자유 소프트웨어입니다. 소프트웨어의 피양도자는 자유 소프트웨어 재단이
 * 공표한 GNU 약소 일반 공중 사용 허가서 2.1판 또는 그 이후 판을 임의로 선택해서, 그 규정에
 * 따라 라이브러리를 개작하거나 재배포할 수 있습니다.
 * 
 * 이 라이브러리는 유용하게 사용될 수 있으리라는 희망에서 배포되고 있지만, 특정한 목적에
 * 맞는 적합성 여부나 판매용으로 사용할 수 있으리라는 묵시적인 보증을 포함한 어떠한 형태의
 * 보증도 제공하지 않습니다. 보다 자세한 사항에 대해서는 GNU 약소 일반 공중 사용 허가서를
 * 참고하시기 바랍니다.
 * 
 * GNU 약소 일반 공중 사용 허가서는 이 라이브러리와 함께 제공됩니다. 만약, 이 문서가 누락
 * 되어있다면 자유 소프트웨어 재단으로 문의하시기 바랍니다.
 * 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * Korean Translation Comes from: http://korea.gnu.org/people/chsong/copyleft/lgpl.ko.html
 * */

using System;
using System.Runtime.InteropServices;

/*
 * Important Note:
 * 상속 관계를 가지는 클래스 사이에서 자식 클래스에만 StructLayoutAttribute 속성을 적용하는 경우
 * 라이브러리를 호스트하는 응용프로그램에서 TypeLoadException이 발생할 수 있습니다. 이 때에는
 * 자식 클래스 뿐만이 아니라 상위 부모 클래스에도 StructLayoutAttribute 속성을 지정해야 합니다.
 */

namespace Blend.xPlatform
{
    /// <summary>
    /// 플랫폼 호출 서비스에서 사용할 구조체에 대한 기본 서비스를 정의하는 추상 클래스입니다.
    /// </summary>
    [Serializable]
    [StructLayout(LayoutKind.Sequential)]
    public abstract class NativeStructureBase : INativeType
    {
        /// <summary>
        /// 구조체의 크기를 가져옵니다.
        /// </summary>
        /// <returns>구조체나 공용체의 크기가 바이트 단위로 반환됩니다.</returns>
        public virtual int SizeOf()
        {
            return Marshal.SizeOf(this.GetType());
        }

        /// <summary>
        /// 크기 값을 받아서 정수 포인터를 반환하는 일반적인 메모리 할당 함수들을 이용하여
        /// 현재 사용하는 구조체를 마샬링합니다.
        /// </summary>
        /// <param name="memoryAllocator">크기 값을 받아서 정수 포인터를 반환하는 함수를 지정합니다.</param>
        /// <returns>마샬링된 결과를 가리키는 비 관리 영역의 정수 포인터가 반환됩니다.</returns>
        public virtual IntPtr ComposeMemoryBlock(Converter<int, IntPtr> memoryAllocator)
        {
            IntPtr memoryBlock = memoryAllocator(this.SizeOf());
            Marshal.StructureToPtr(this, memoryBlock, true);
            return memoryBlock;
        }

        /// <summary>
        /// 정수 포인터를 받아서 연결된 메모리 개체를 해제합니다.
        /// </summary>
        /// <param name="memoryRemover">할당 해제 함수를 지정합니다.</param>
        /// <param name="targetAddress">할당 해제할 객체를 가리키는 정수 포인터를 지정합니다.</param>
        public virtual void DisposeMemoryBlock(Action<IntPtr> memoryRemover, IntPtr targetAddress)
        {
            memoryRemover(targetAddress);
        }
    }
}
