﻿/*
 * Project Blend: the Blossom of .NET Development
 * Copyright (c) 2007 Project Blend
 * 
 * 이 라이브러리는 자유 소프트웨어입니다. 소프트웨어의 피양도자는 자유 소프트웨어 재단이
 * 공표한 GNU 약소 일반 공중 사용 허가서 2.1판 또는 그 이후 판을 임의로 선택해서, 그 규정에
 * 따라 라이브러리를 개작하거나 재배포할 수 있습니다.
 * 
 * 이 라이브러리는 유용하게 사용될 수 있으리라는 희망에서 배포되고 있지만, 특정한 목적에
 * 맞는 적합성 여부나 판매용으로 사용할 수 있으리라는 묵시적인 보증을 포함한 어떠한 형태의
 * 보증도 제공하지 않습니다. 보다 자세한 사항에 대해서는 GNU 약소 일반 공중 사용 허가서를
 * 참고하시기 바랍니다.
 * 
 * GNU 약소 일반 공중 사용 허가서는 이 라이브러리와 함께 제공됩니다. 만약, 이 문서가 누락
 * 되어있다면 자유 소프트웨어 재단으로 문의하시기 바랍니다.
 * 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * Korean Translation Comes from: http://korea.gnu.org/people/chsong/copyleft/lgpl.ko.html
 * */

using System;
using System.IO;
using System.Text;
using Blend.Sortings;
using System.Reflection;
using System.Windows.Forms;
using System.Collections.Generic;

namespace Blend.xPlatform.Development.Mapper
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            this.InitializeComponent();
        }

        private void browseTargetAssembly_Click(object sender, EventArgs e)
        {
            DialogResult dialogResult = this.browseAssemblyDialog.ShowDialog();

            if (!dialogResult.Equals(DialogResult.OK))
                return;

            if (!File.Exists(this.browseAssemblyDialog.FileName))
                return;

            this.targetAssemblyPath.Text = this.browseAssemblyDialog.FileName;
        }

        private void browseFunctionNames_Click(object sender, EventArgs e)
        {
            DialogResult dialogResult = this.browseFunctionNameDialog.ShowDialog();

            if (!dialogResult.Equals(DialogResult.OK))
                return;

            if (this.browseFunctionNameDialog.FileNames == null ||
                this.browseFunctionNameDialog.FileNames.Length < 1)
                return;

            this.functionNamesPath.Text = this.browseFunctionNameDialog.FileName;
        }

        private void exitButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void clearButton_Click(object sender, EventArgs e)
        {
            this.results.Clear();
        }

        private void analyzeButton_Click(object sender, EventArgs e)
        {
            this.results.Clear();

            if (!File.Exists(this.targetAssemblyPath.Text))
                return;

            if (!File.Exists(this.functionNamesPath.Text))
                return;

            Assembly targetAssembly = Assembly.LoadFile(this.targetAssemblyPath.Text);

            if (targetAssembly == null)
                return;

            Type[] typeArray = targetAssembly.GetTypes();
            List<Type> typeCandidates = new List<Type>();

            if (typeArray == null || typeArray.Length < 1)
                return;

            foreach (Type eachType in typeArray)
            {
                if (eachType == null || !eachType.IsSubclassOf(typeof(PlatformInvokeBase)))
                    continue;

                typeCandidates.Add(eachType);
            }

            Type selectedType = null;

            if (typeCandidates.Count > 1)
            {
                using (TypeSelectForm form = new TypeSelectForm(typeCandidates))
                {
                    DialogResult dialogResult = form.ShowDialog();

                    if (!dialogResult.Equals(DialogResult.OK))
                        return;

                    selectedType = form.SelectedType;
                }
            }
            else
                selectedType = typeCandidates[0] as Type;

            if (selectedType == null)
                return;

            string[] lines = File.ReadAllLines(this.functionNamesPath.Text, Encoding.Default);

            if (lines == null || lines.Length < 1)
                return;

            string temp = null;
            List<string> filteredLines = new List<string>();

            foreach (string eachLine in lines)
            {
                temp = eachLine.Trim();

                if (temp.Length < 1)
                    continue;

                if (filteredLines.Contains(temp))
                    continue;

                filteredLines.Add(temp);
            }

            lines = filteredLines.ToArray();

            SortBase<string> foundNames = new QuickSort<string>();
            SortBase<string> notFoundNames = new QuickSort<string>();
            SortBase<string> missingNames = new QuickSort<string>();
            MethodInfo[] targetMethods = selectedType.GetMethods();

            foreach (MethodInfo eachMethod in targetMethods)
            {
                if (eachMethod == null)
                    continue;

                if (((int)(eachMethod.Attributes & MethodAttributes.PinvokeImpl)).Equals(0))
                    continue;

                if (Array.IndexOf<string>(lines, eachMethod.Name) < 0 &&
                    !notFoundNames.Contains(eachMethod.Name))
                    notFoundNames.Add(eachMethod.Name);
                else if (!foundNames.Contains(eachMethod.Name))
                    foundNames.Add(eachMethod.Name);
            }

            foreach (string eachRawLine in lines)
            {
                string eachLine = eachRawLine.Trim();

                if (eachLine == null || eachLine.Length.Equals(0))
                    continue;

                if (eachLine[0].Equals('$') || // $I10_OUTPUT 같은 형태의 이름
                    eachLine[0].Equals('?') || // Microsoft C++ 스타일 이름 꾸미기 식별자
                    eachLine[0].Equals('#') || // 주석
                    Char.IsDigit(eachLine[0]) ||
                    eachLine.Equals("N/A", StringComparison.CurrentCultureIgnoreCase))
                    continue;

                try
                {
                    if (selectedType.GetMethod(eachLine) == null &&
                        !missingNames.Contains(eachLine))
                    {
                        missingNames.Add(eachLine);
                        continue;
                    }

                    if (!foundNames.Contains(eachLine))
                        foundNames.Add(eachLine);
                }
                catch (AmbiguousMatchException)
                {
                    if (!foundNames.Contains(eachLine))
                        foundNames.Add(eachLine);
                }
            }

            if (missingNames.Count > 0)
            {
                int index = (-1);
                PropertyInfo[] targetProperties = selectedType.GetProperties();
                FieldInfo[] targetFields = selectedType.GetFields();

                foreach (PropertyInfo eachProperty in targetProperties)
                {
                    index = missingNames.IndexOf(eachProperty.Name);

                    if (index > (-1))
                    {
                        if (!foundNames.Contains(missingNames[index]))
                            foundNames.Add(missingNames[index]);

                        missingNames.RemoveAt(index);
                        continue;
                    }
                }

                foreach (FieldInfo eachField in targetFields)
                {
                    index = missingNames.IndexOf(eachField.Name);

                    if (index > (-1))
                    {
                        if (!foundNames.Contains(missingNames[index]))
                            foundNames.Add(missingNames[index]);

                        missingNames.RemoveAt(index);
                        continue;
                    }
                }
            }

            foundNames.Sort();
            missingNames.Sort();
            notFoundNames.Sort();

            StringBuilder buffer = new StringBuilder(4096);

            if (foundNames.Count > 0)
            {
                buffer.AppendLine(String.Format("Found Functions/Symbols ({0} identifier/s...): ", foundNames.Count));

                foreach (string eachLine in foundNames)
                    buffer.AppendLine(eachLine);

                buffer.AppendLine();
            }

            if (missingNames.Count > 0)
            {
                buffer.AppendLine(String.Format("Missing Functions/Symbols ({0} identifier/s...): ", missingNames.Count));

                foreach (string eachLine in missingNames)
                    buffer.AppendLine(eachLine);

                buffer.AppendLine();
            }

            if (notFoundNames.Count > 0)
            {
                buffer.AppendLine(String.Format("Not Found Functions/Symbols (Maybe invalid P/Invoke, {0} identifier/s...): ", notFoundNames.Count));

                foreach (string eachLine in notFoundNames)
                    buffer.AppendLine(eachLine);

                buffer.AppendLine();
            }

            this.results.Text = buffer.ToString();
        }

        private void copyButton_Click(object sender, EventArgs e)
        {
            if (this.results.Text.Length > 0)
                Clipboard.SetText(this.results.Text);
        }
    }
}